#include "montecarlo.h"
#include <QDebug>

MonteCarlo::MonteCarlo() {
    _step = 0;
}

bool MonteCarlo::step(QVector<QVector<Cell> > &src, QVector<QVector<Cell> > &dest, bool boundaryPeriodic, QVector<CellLocation> &emptyCells) {
    if (!_step) {
        for(int x = 0; x < src.count(); x++) {
            for(int y = 0; y < src[x].count(); y++) {
                dest[x][y].setGrainId(rand() % 10);
            }
        }
    }
    else if (_step < 100) {
        _montecarlo(src, dest, boundaryPeriodic, emptyCells);
    }
    else {
        return true;
    }
    _step++;
    return false;
}


void MonteCarlo::_montecarlo(QVector<QVector<Cell> > &src, QVector<QVector<Cell> > &dest, bool boundaryPeriodic, QVector<CellLocation> &emptyCells) {
    QVector<CellLocation> onBorder;
    QVector<CellLocation> neighbors;

    _updateBorders(src);
    _cellsOnBorder(src, onBorder);

    while(!onBorder.isEmpty()) {
        struct CellLocation loc;
        int idx = rand() % onBorder.count();
        loc = onBorder[idx];
        onBorder.remove(idx);

        //qDebug() << "x =" << loc.x << ", y =" << loc.y << ", count =" << onBorder.count();
        neighbors.clear();

        if (!src[loc.x][loc.y].isEmpty()) {
            double energy, newenergy;
            struct CellLocation newLoc;
            int newId;
            neighbors.clear();
            _getMatchingNeighbors(src, loc.x, loc.y, true, false, neighbors);

            if (neighbors.isEmpty())
                continue;

            energy = neighbors.count();
            newenergy = neighbors.count();

            newLoc = neighbors[rand() % neighbors.count()];
            newId = src[newLoc.x][newLoc.y].getGrainId();

            for(int i = 0; i < neighbors.count(); i++) {
                struct CellLocation nLoc;
                nLoc = neighbors[i];

                if (src[nLoc.x][nLoc.y].getGrainId() == src[loc.x][loc.y].getGrainId()) {
                    energy -= 1.0;
                }

                if (src[nLoc.x][nLoc.y].getGrainId() == newId) {
                    newenergy -= 1.0;
                }
            }

            //qDebug() << "energy =" << energy << "newEnergy =" << newenergy;

            if ((newenergy <= energy) && (newId > 0)) {

                //qDebug() << "newId =" << newId;
                dest[loc.x][loc.y].setGrainId(newId);
                dest[loc.x][loc.y].setEnergy(newenergy);
            }
        }
    }

    _updateBorders(dest);
}
