#include "simulation.h"


void Simulation::_checkAndSetBorder(int x, int y, QVector<QVector<Cell> > &cells) {
    int id;
    QVector<CellLocation> neighbors;

    _getAllNeighbors(cells, x, y, neighbors);
    id = cells[x][y].getGrainId();

    cells[x][y].setOnBorder(false);

    for(int i = 0; i < neighbors.count(); i++) {
        struct CellLocation loc = neighbors[i];
        if (cells[loc.x][loc.y].getGrainId() != id) {
            cells[x][y].setOnBorder(true);
            break;
        }
    }
}

void Simulation::_updateBorders(QVector<QVector<Cell> > &cells) {
    for(int x = 0; x < cells.count(); x++) {
        for(int y = 0; y < cells[x].count(); y++) {
            _checkAndSetBorder(x, y, cells);
        }
    }
}

void Simulation::_getAllNeighbors(QVector<QVector<Cell> > &cells, int x, int y, QVector<CellLocation> &neighbors) {
    for(int ix = x - 1; ix <= x + 1; ix++) {
        struct CellLocation loc;

        if ((ix < 0) || (ix >= cells.count()))
            continue;

        for(int iy = y - 1; iy <= y + 1; iy++) {
            if ((iy < 0) || (iy >= cells[ix].count()))
                continue;

            loc.x = ix;
            loc.y = iy;
            neighbors.append(loc);
        }
    }
}

void Simulation::_cellsOnBorder(QVector<QVector<Cell> > &cells, QVector<CellLocation> &onBorder) {
    struct CellLocation loc;
    for(int x = 0; x < cells.count(); x++) {
        for(int y = 0; y < cells[x].count(); y++) {
            if (cells[x][y].onBorder()) {
                loc.x = x;
                loc.y = y;
                onBorder.append(loc);
            }
        }
    }
}

void Simulation::_getMatchingNeighbors(QVector<QVector<Cell> > &cells, int x, int y, bool willGrown, bool recristalized, QVector<CellLocation> &matching) {
    QVector<CellLocation> neighbors;

    _getAllNeighbors(cells, x, y, neighbors);

    for(int i = 0; i < neighbors.count(); i++) {
        struct CellLocation loc = neighbors[i];
        if (cells[loc.x][loc.y].willGrown() == willGrown) {
            matching.append(loc);
        }
    }
}

int Simulation::_calcutateNeighbors(QVector<QVector<Cell> > &cells, int x, int y, bool mask[3][3], int *resultId) {
    int id, max;
    int counters[255];
    memset(&counters, 0, sizeof(counters));

    for(int ix = x - 1, mx = 0; ix <= x + 1; ix++, mx++) {
        for(int iy = y - 1, my = 0; iy <= y + 1; iy++, my++) {
            if ((ix < 0) || (iy < 0) || (ix >= cells.count()) || (iy >= cells[ix].count()) || (!mask[mx][my]))
                continue;

            if (cells[ix][iy].isEmpty()) {
                continue;
            }

            id = cells[ix][iy].getGrainId();
            counters[id]++;
        }
    }

    id = 0;
    max = 0;

    for(int i = 0; i < 255; i++) {
        if (counters[i] > max) {
            max = counters[i];
            id = i;
        }
    }

    *resultId = id;
    return max;
}
