#include "srx.h"
#include <QDebug>

void SRX::_addNucletions(int count, QVector<QVector<Cell> > &cells) {
    QVector<CellLocation> onBorder;

    _cellsOnBorder(cells, onBorder);

    for(int i = 0; i < count; i++) {
        //int idx = rand() & onBorder.count();
        //int x = onBorder[idx].x;
        //int y = onBorder[idx].y;
        int x = rand() % cells.count();
        int y = rand() % cells[x].count();

        if (_canRecrystalize(x, y, cells)) {
            //qDebug() << "SRX: Can Recrystalize";
            cells[x][y].setGrainId(i);
            cells[x][y].setWillGrown(true);
            cells[x][y].setRecristalized(true);
        }
        else {
            i--;
        }
    }
}

bool SRX::_canRecrystalize(int x, int y, QVector<QVector<Cell> > &cells) {
    double energy = _getNeighboursEnergy(x, y, cells);
    //qDebug() << "energy =" << energy;
    if (energy / 8 > 10 / 8) {
        cells[x][y].setEnergy(energy);
        return true;
    }

    return false;
}

double SRX::_getNeighboursEnergy(int x, int y, QVector<QVector<Cell> > &cells) {
    QVector<CellLocation> neighbors;
    struct CellLocation loc;
    _getAllNeighbors(cells, x, y, neighbors);
    double energy = 0.0;

    for(int i = 0; i < neighbors.count(); i++) {
        loc = neighbors[i];
        energy += cells[loc.x][loc.y].getEnergy();
    }

    return energy;
}

bool SRX::_MCSRX(int x, int y, QVector<QVector<Cell> > &cells) {
    //qDebug() << "_MCSRX";
    int count, id;
    static bool mask[3][3] = {
        { true, true, true },
        { true, false, true },
        { true, true, true }
    };

    if (cells[x][y].getGrainId() > 0) {
        double cellEnergy = cells[x][y].getEnergy();
        double tmpEnergy = _getNeighboursEnergy(x, y, cells);
        cellEnergy += tmpEnergy;

        if (rand() % 2 == 0) {
            count = _calcutateNeighbors(cells, x, y, mask, &id);
            //qDebug() << "id =" << id;
            cells[x][y].setGrainId(id);
            //cells[x][y].setEnergy(cellEnergy);
            cells[x][y].setEnergy(0);

            //qDebug() << "_OK";
            return true;
        }
    }
    return false;
}

SRX::SRX() {
}

bool SRX::step(QVector<QVector<Cell> > &src, QVector<QVector<Cell> > &dest, bool boundaryPeriodic, QVector<CellLocation> &emptyCells) {
    //qDebug() << "SRX: Step!";
    static int __step = 0;

    if ((__step % 5) == 0) {
        _addNucletions(10, dest);
    }
    __step++;

    int iter = 0;

    while(iter < (dest.count() * dest[0].count()) - (dest.count() + dest[0].count()) * 2) {
        int x = rand() % dest.count();
        int y = rand() % dest[x].count();

        if (_MCSRX(x, y, dest)) iter++;
    }

    return false;
}
