#include "wndmain.h"
#include "ui_wndmain.h"
#include "class1.h"
#include "class3.h"
#include "montecarlo.h"
#include "srx.h"
#include <stdlib.h>
#include <time.h>
#include <QMessageBox>
#include <QDebug>
#include "wndinclusions.h"

WndMain::WndMain(QWidget *parent) : QMainWindow(parent), _ui(new Ui::WndMain) {
    _ui->setupUi(this);

    srand((unsigned int)time(NULL));

    _space = NULL;
    _simulation = NULL;
    _previewScene = new QGraphicsScene();
    _ui->graphicsView->setScene(_previewScene);

    connect(_ui->pbReset, SIGNAL(clicked()), this, SLOT(_reset()));
    connect(_ui->pbStart, SIGNAL(clicked()), this, SLOT(_start()));
    connect(_ui->pbClass1, SIGNAL(clicked()), this, SLOT(_class1()));
    connect(_ui->pbClass3, SIGNAL(clicked()), this, SLOT(_class3()));
    connect(_ui->pbMC, SIGNAL(clicked()), this, SLOT(_mc()));
    connect(_ui->pbSRX, SIGNAL(clicked()), this, SLOT(_srx()));
    connect(_ui->pbInclusions, SIGNAL(clicked()), this, SLOT(_inclusions()));
    connect(_ui->pbDrawEnergy, SIGNAL(clicked()), this, SLOT(_drawEnergy()));
    connect(_ui->pbEnergy, SIGNAL(clicked()), this, SLOT(_distributeEnergy()));
    connect(_ui->pbRedraw, SIGNAL(clicked()), this, SLOT(_drawSpace()));
}

WndMain::~WndMain() {
    if (_space)
        delete _space;

    delete _ui;
    delete _previewScene;
}

void WndMain::changeEvent(QEvent *e) {
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        _ui->retranslateUi(this);
        break;
    default:
        break;
    }
}


void WndMain::_reset(){
    if (_space)
        delete _space;

    _space = new Space(_ui->sbWidth->value(), _ui->sbHeight->value(), _ui->cbBoundary->currentIndex() == 1);
    _space->draw(_previewScene);
}

void WndMain::_start() {
    if (!_simulation) {
        QMessageBox::information(this, "Can't start", "Select Class first");
        return;
    }

    int iter = 1;
    while(!_space->simulationStep()) {
        _ui->statusBar->showMessage(QString("Iteration %1").arg(iter++));
        _space->draw(_previewScene);
        QApplication::processEvents();
        //break;
    }

    _space->draw(_previewScene);
}

void WndMain::_class1() {
    if (!_space) {
        QMessageBox::information(this, "Can't select class", "Create space first");
        return;
    }

    if (_simulation) {
        delete _simulation;
    }

    _simulation = new Class1Simulation((Neighborhood)(_ui->cbNeighborhood->currentIndex() + 1));
    _space->setSimulation(_simulation);
    _space->draw(_previewScene);
    _addStartPoints();
}

void WndMain::_class3() {
    if (!_space) {
        QMessageBox::information(this, "Can't select class", "Create space first");
        return;
    }

    if (_simulation) {
        delete _simulation;
    }

    _simulation = new Class3Simulation();
    _space->setSimulation(_simulation);
    _space->draw(_previewScene);
    _addStartPoints();
}

void WndMain::_mc() {
    if (!_space) {
        QMessageBox::information(this, "Error", "Create space first");
        return;
    }

    if (_simulation) {
        delete _simulation;
    }

    _simulation = new MonteCarlo();
    _space->setSimulation(_simulation);
    _space->draw(_previewScene);
}

void WndMain::_srx() {
    if (!_space) {
        QMessageBox::information(this, "Error", "Create space first");
        return;
    }

    if (_simulation) {
        delete _simulation;
    }

    _simulation = new SRX();
    _space->setSimulation(_simulation);
    _space->draw(_previewScene);

    int iter = 0;
    while(++iter < _ui->sbIterations->value()) {
        _space->simulationStep();
        _ui->statusBar->showMessage(QString("Iteration %1").arg(iter++));
        _space->draw(_previewScene);
        QApplication::processEvents();
        //break;
    }

    _space->draw(_previewScene);
}

void WndMain::_drawEnergy() {
    _space->drawEnergy(_previewScene);
}

void WndMain::_drawSpace() {
    _space->draw(_previewScene);
}

void WndMain::_distributeEnergy() {
    _space->setGrainEnergy(_ui->sbGrainEnergy->value(), _ui->cbDistribution->currentIndex());
}

void WndMain::_inclusions() {
    WndInclusions wnd;
    int x, y;
    QVector<struct Inclusion> list;

    if (wnd.exec() != QDialog::Accepted)
        return;

    list = wnd.getInclusionList();
    for(int i = 0; i < list.count(); i++) {
        x = rand() % _space->width();
        y = rand() % _space->height();

        if (_space->simulationFinished()) { /* Tylko na granicach */
            qDebug() << "add on border";
            while (!_space->isCellOnBorder(x, y)) {
                x = rand() % _space->width();
                y = rand() % _space->height();
            }
        }

        _space->addInclusion(x, y, list[i]);
    }

    _space->draw(_previewScene);
}

void WndMain::_addStartPoints() {
    int i, x, y;
    if (_ui->cbRandomStart->isChecked()) {
        i = 0;
        while(i < _ui->sbPhases->value()) {
            x = rand() % _ui->sbWidth->value();
            y = rand() % _ui->sbHeight->value();

            if (_space->setCellId(x, y, i + 1))
                i++;
        }
    }
    else {

    }

    _space->draw(_previewScene);
}
